@@ -3,6 +3,10 @@
 
 #include "BigMatrix.h"
 
+//TODO: It would be nice to have accessors handle row and column offsets
+//however, the only way to do this would be with a proxy.  Is there another
+//solution so that the offsets don't have to go in Cbigmemory.cpp?
+
 // The BigMatrixAccessor class allows the user to access non-separated
 // big matrix data as matrix[i][j].
 template<typename T>
@@ -17,16 +21,15 @@ class BigMatrixAccessor
 
     BigMatrixAccessor( BigMatrix &bm )
     {
-      _pMat = reinterpret_cast<T*>(bm.matrix());
-      _nrow = bm.nrow();
+      _pMat = reinterpret_cast<T*>(reinterpret_cast<T*>(bm.matrix()));
+      _nrow = bm.num_rows();
     };
 
-    // Is the bracket operator fast enough?  Do we need to inline this?
     inline T* operator[](const unsigned long col) {return _pMat+_nrow*col;};
 
   protected:
     T *_pMat;
-    unsigned long _nrow;
+    long _nrow;
 };
 
 template<typename T>
